unit BrowserFormUnit;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.WebBrowser,
  FMX.StdCtrls, FMX.Edit, FMX.Controls.Presentation, FMX.Objects, FMX.Layouts,
  System.IOUtils, System.StrUtils, System.Messaging,
  SimpleGateUnit;

type
  TBrowserForm = class(TForm)
    Layout1: TLayout;
    Rectangle1: TRectangle;
    Label1: TLabel;
    Edit1: TEdit;
    EditButton1: TEditButton;
    StatusBar1: TStatusBar;
    Label2: TLabel;
    Layout2: TLayout;
    procedure EditButton1Click(Sender: TObject);
    procedure WebBrowser1DidFinishLoad(ASender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CreateBrowser;
    procedure MessageReceiver(const Sender: TObject; const M: TMessage);
  private
    WebBrowser1: TWebBrowser;
    FSimpleGate: TSimpleGate;
  public
    procedure BrowserFormShow(Url: string; Hint: string);
    procedure ShowVideo(Filename: string);
    procedure VideoMetaData(Text: string);
  end;

var
  BrowserForm: TBrowserForm;

implementation

uses
  TTwitterBot2ClassUnit, TwitterAuthorizeUnit;

{$R *.fmx}

procedure TBrowserForm.EditButton1Click(Sender: TObject);
begin
  if not Edit1.Text.IsEmpty then
    WebBrowser1.Navigate(Edit1.Text);
end;

procedure TBrowserForm.FormCreate(Sender: TObject);
var
  SubscriptionId: Integer;
  MessageManager: TMessageManager;
begin
  WebBrowser1 := nil;
  FSimpleGate := nil;

  MessageManager := TMessageManager.DefaultManager;
  SubscriptionId := MessageManager.SubscribeToMessage(TMessage<UnicodeString>, MessageReceiver);
end;

procedure TBrowserForm.FormDestroy(Sender: TObject);
begin
  if Assigned(WebBrowser1) then
    WebBrowser1.Free;
  if Assigned(FSimpleGate) then
    FSimpleGate.Free;
end;

procedure TBrowserForm.WebBrowser1DidFinishLoad(ASender: TObject);
var
  MessageManager: TMessageManager;
  Message: TMessage;
begin
  if WebBrowser1.URL.StartsWith('http', true) then begin
      MessageManager := TMessageManager.DefaultManager;
      Message := TMessage<UnicodeString>.Create(WebBrowser1.URL);
      MessageManager.SendMessage(ASender, Message, True);
  end;
end;

procedure TBrowserForm.MessageReceiver(const Sender: TObject; const M: TMessage);
var
  AText: string;
  DoCloseWebForm: boolean;
begin
  AText := (M as TMessage<UnicodeString>).Value;
  if AText.StartsWith('http', true) then begin
    Edit1.Text := AText;
    TwitterAuthorizeForm.UrlChanged(AText);
  end;
end;



procedure TBrowserForm.BrowserFormShow(Url: string; Hint: string);
begin
  if not Url.IsEmpty then begin
    Self.Visible := true;
    if not Assigned(WebBrowser1) then
      CreateBrowser;
    WebBrowser1.Navigate(Url);
    Label2.Text := Hint;
  end else
    Self.Visible := false;
end;

procedure TBrowserForm.CreateBrowser;
begin
  WebBrowser1 := TWebBrowser.Create(Self);
  WebBrowser1.Parent := Layout2;
  {$IFDEF MSWINDOWS}
  WebBrowser1.WindowsEngine := TWindowsEngine.EdgeOnly;
  {$ENDIF}
  WebBrowser1.Align := TAlignLayout.Client;
  WebBrowser1.Visible := true;
  WebBrowser1.OnDidFinishLoad := WebBrowser1DidFinishLoad;
end;

function LoadTextFromResource(AName: string): string;
var
  Stream: TResourceStream;
  SList: TStringList;
begin
  result := '';
  try
    Stream := TResourceStream.Create(HInstance, AName, RT_RCDATA);
    if Stream <> nil then begin
      try
        if Stream.Size > 0 then begin
          Stream.Seek(0, TSeekOrigin.soBeginning);
          SList := TStringList.Create;
          try
            SList.LoadFromStream(Stream, TEncoding.UTF8);
            result := SList.Text;
          finally
            SList.Free;
          end;
        end;
      finally
        Stream.Free;
      end;
    end;
  except

  end;
end;

procedure TBrowserForm.ShowVideo(Filename: string);
begin
  if not Filename.IsEmpty then begin
    Self.Visible := true;
    if not Assigned(WebBrowser1) then
      CreateBrowser;
    if not Assigned(FSimpleGate) then
     FSimpleGate := TSimpleGate.Create(Self, WebBrowser1, 'delphi', 0);



    try
      var Html := LoadTextFromResource('PlayLocalVideoHtml');
      if not Html.IsEmpty then begin
        Html := Html.Replace('VIDEOFILENAME', 'file://' + Filename);
        var HtmlFilename := AppDir + PathDelim + 'PlayLocalVideo.html';
        Edit1.Text := HtmlFilename;
        TFile.WriteAllText(HtmlFilename, Html, TEncoding.UTF8);
        WebBrowser1.Navigate(HtmlFilename);
      end;
    except
      on e: exception do
        AddToLog('TBrowserForm.ShowVideo: ' + e.Message);
    end;
  end;

end;

procedure TBrowserForm.VideoMetaData(Text: string);
begin
  var SArray := SplitString(Text, '_');
  TThread.CreateAnonymousThread(procedure
    function CutDez(AText: string): string;
    var v: single;
    begin
      result := AText;
      if TryStrToFloat(AText, v) then
        result := v.ToString(TFloatFormat.ffFixed, 7, 1);
    end;
  begin
    if Length(SArray) = 3 then begin
      var s := 'Len=' + CutDez(SArray[0]) + 's, ' + 'W=' + SArray[1] + ', ' + 'H=' + SArray[2];
      TThread.Synchronize(
        TThread.CurrentThread,
        procedure
        begin
          Label2.Text := s;
          TwitterAuthorizeForm.AddVideoMetaData(s);
        end);
    end;
  end).Start;
end;

end.
