//
// by melanion.info
//
// TSimpleGate is for VCL apps
//
// TSimpleGate usage
//
// <script language="javascript" type="text/javascript">
//		function OnButtonClick() {
//      var s = 'button clicked !';
//			window.location.href = 'delphi:MyProc(' + s + ')';     // call delphi
//		}
//    function callFromDelphi() {
//      var resultValue =  ...
//      window.location.href = 'delphi:return=' + resultValue; // = return resultValue
//    }
// </script>
//
// <body>
//   <a href="delphi:MyProc(OK)">call delphi</a>
//

unit SimpleGateVclUnit;

interface

uses
  System.Classes, System.SysUtils, IdURI, System.DateUtils, System.Rtti, Vcl.Edge,
  System.Messaging;

type
  TResultProc = reference to procedure (const iResult: String);

  TSimpleGate = class
  strict private
    FReceiver: TObject;
    FWebBrowser: TEdgeBrowser;
    FScheme: String;
    FTag: integer;
    FResultProc: TResultProc;
    FLastReturnTime: TDateTime;
    FLastReturnString: string;
    FScriptLoaded: boolean;
    FRttiContext: TRttiContext;
    procedure ScriptLoaded;
    function CheckURL(var Url: string): boolean;
    procedure EdgeBrowserNavigationStarting(Sender: TCustomEdgeBrowser;
      Args: TNavigationStartingEventArgs);
    procedure CallEvent(AText: string);
    procedure MessageReceiver(const Sender: TObject; const M: TMessage);

  public
    constructor Create(const Receiver: TObject; const WebBrowser: TEdgeBrowser; const Scheme: String; Tag: integer); //reintroduce;
    destructor Destroy; override;
    procedure CallScript(const Script: String; const ResultProc: TResultProc);
    property Loaded: boolean read FScriptLoaded;
  end;


implementation

uses
  MainFormUnit;

{ TSimpleGate }

// if TSimpleGate is used at the same time, a Tag value must be unique
constructor TSimpleGate.Create(const Receiver: TObject; const WebBrowser: TEdgeBrowser; const Scheme: String; Tag: integer);
var
  SubscriptionId: Integer;
  MessageManager: TMessageManager;
begin
  FReceiver := Receiver;
  FWebBrowser := WebBrowser;
  FScheme := LowerCase(Scheme);
  FTag := Tag;

  FScriptLoaded := false;

  FWebBrowser.OnNavigationStarting := EdgeBrowserNavigationStarting;
  FResultProc := nil;
  FLastReturnString := '';
  FLastReturnTime := 0;

  FRttiContext := TRttiContext.Create;

  MessageManager := TMessageManager.DefaultManager;
  SubscriptionId := MessageManager.SubscribeToMessage(TMessage<UnicodeString>, MessageReceiver);
end;

destructor TSimpleGate.Destroy;
begin
  try
    FRttiContext.Free;
  except

  end;

  inherited;
end;

procedure TSimpleGate.MessageReceiver(const Sender: TObject; const M: TMessage);
var
  AText: string;
begin
  if AppClosing then exit;

  AText := (M as TMessage<UnicodeString>).Value;
  if SameText(AText, 'ScriptLoaded_' + FTag.ToString) then begin
    ScriptLoaded;
  end else if AText.EndsWith('_' + FTag.ToString) then begin
    AText := AText.Remove(Length(AText) - 2);
    CallEvent(AText);
  end;
end;

procedure TSimpleGate.ScriptLoaded;
begin
  FScriptLoaded := true;
end;

function TSimpleGate.CheckURL(var Url: string): boolean;
begin
  result := false;

  Url := TIDURI.URLDecode(Url);
  if Url.StartsWith('delphi:', true) and
     (SecondsBetween(FLastReturnTime, Now) > 1) and not SameText(FLastReturnString, Url) then begin

    FLastReturnString := Url;
    FLastReturnTime := now;

    Url := Trim(Url.Substring(FScheme.Length + 1));
    if not Url.IsEmpty then
      result := true;
  end;
end;

procedure TSimpleGate.EdgeBrowserNavigationStarting(Sender: TCustomEdgeBrowser;
      Args: TNavigationStartingEventArgs);
var
  PUrl: PChar;
  Url: string;
  MessageManager: TMessageManager;
  Message: TMessage;
begin
  PUrl := StrAlloc(2048);  // Max URL length
  Args.ArgsInterface.Get_uri(PUrl);
  StrPCopy(PUrl, Url);
  if CheckURL(Url) then begin
    if Url.StartsWith('return=', true) then begin
      if Assigned(FResultProc) then begin
        FResultProc(Url.Substring(Length('return=')));
        FResultProc := nil;
      end;
    end else if not FScriptLoaded and Url.StartsWith('ScriptLoaded', true) then begin
      FScriptLoaded := true;
      MessageManager := TMessageManager.DefaultManager;
      Message := TMessage<UnicodeString>.Create('ScriptLoaded_' + FTag.ToString);
      MessageManager.SendMessage(Sender, Message, True);
    end else begin
      // CallEvent(AUrl);
      MessageManager := TMessageManager.DefaultManager;
      Message := TMessage<UnicodeString>.Create(Url + '_' + FTag.ToString);
      MessageManager.SendMessage(Sender, Message, True);
    end;
    Args.ArgsInterface.Set_Cancel(1);
  end;
end;

// from SG.ScriptGate.pas by HOSOKAWA
procedure TSimpleGate.CallEvent(AText: string);
begin
  TThread.CreateAnonymousThread(procedure
    const
      SCHEME_SEP = ':';
      METHOD_START_SEP = '(';
      METHOD_END_SEP = ')';
      PARAM_SEP = ',';
      SQUOTE = '''';
      DQUOTE = '"';
      SEMICOLON = ';';
    var
      iURL: string;
      Factors: TArray<String>;
      FactorsLen: Integer;
      Method: String;
      i: Integer;
      RttiType: TRttiType;
      RttiMethod: TRttiMethod;
      RttiParams: TArray<TRttiParameter>;
      VarParams: TArray<TValue>;
      ParamsLen: Integer;
      Factor: String;
    begin
      iURL := AText;
      Factors := iURL.Split([METHOD_START_SEP]);
      Method := Factors[0];

      iURL := iURL.Substring(Method.Length + 1);
      Factors :=
        iURL.Split(
          [PARAM_SEP, METHOD_END_SEP],
          TStringSplitOptions.ExcludeEmpty);

      RttiType := FRttiContext.GetType(FReceiver.ClassType);
      if RttiType = nil then exit;

      RttiMethod := RttiType.GetMethod(Method);
      if RttiMethod = nil then exit;

      FactorsLen := Length(Factors);
      RttiParams := RttiMethod.GetParameters;
      ParamsLen := Length(RttiParams);

      if ParamsLen <> FactorsLen then exit;

      SetLength(VarParams, Length(RttiParams));
      for i := Low(RttiParams) to High(RttiParams) do begin
        Factor := Factors[i].TrimRight;

        if (Factor.EndsWith(SEMICOLON)) then
          Factor := Factor.Substring(0, Factor.Length - 1).TrimRight;

        if (Factor.EndsWith(METHOD_END_SEP)) then
          Factor := Factor.Substring(0, Factor.Length - 1);

        Factor := Factor.Trim;
        Factor := Factor.DeQuotedString(SQUOTE);
        Factor := Factor.DeQuotedString(DQUOTE);

        case (RttiParams[i].ParamType.TypeKind) of
          TTypeKind.tkInteger,
          TTypeKind.tkInt64,
          TTypeKind.tkClass,
          TTypeKind.tkClassRef,
          TTypeKind.tkInterface,
          TTypeKind.tkPointer:
            try
              VarParams[i] := Factor.ToInteger;
            except
              VarParams[i] := 0;
            end;

          TTypeKind.tkFloat:
            VarParams[i] := Factor.ToDouble;

          TTypeKind.tkEnumeration:
            try
              VarParams[i] := Factor.ToBoolean;
            except
              VarParams[i] := Factor;
            end;

        else
          VarParams[i] := Factor;
        end;
      end;


      TThread.Synchronize(
        TThread.CurrentThread,
        procedure
        begin
          try
            RttiMethod.Invoke(FReceiver, VarParams);
          except

          end;
        end
      );

    end).Start;
end;

procedure TSimpleGate.CallScript(const Script: String; const ResultProc: TResultProc);
begin
  FResultProc := ResultProc;
  FWebBrowser.ExecuteScript(Script);
end;

end.
